
# Messages
DISCONNECTED = u'DISCONNECTED'
NICK = u'NICK'
USER = u'USER'
PING = u'PING'
PONG = u'PONG'
JOIN = u'JOIN'
PART = u'PART'
QUIT = u'QUIT'
PRIVMSG = u'PRIVMSG'
NOTICE = u'NOTICE'
MODE = u'MODE'
INVITE = u'INVITE'

# Pseudo-messages
# socket level
CREATED = u'CREATED'
CONNECTED = u'CONNECTED'
# event level
LOADED = u'LOADED'

# Message-like
ERROR = u'ERROR'

# From RFC
# sed -n -e "s/ *\([0-9][0-9][0-9]\)  *\([A-Z][A-Z]*_[A-Z][A-Z_]*\)/\2 = u'\1'/p"
# :s/\(.*\) \(.*\)/\2 = u'\1'/
RPL_WELCOME = u'001'
RPL_YOURHOST = u'002'
RPL_CREATED = u'003'
RPL_MYINFO = u'004'
RPL_BOUNCE = u'005'
RPL_USERHOST = u'302'
RPL_ISON = u'303'
RPL_AWAY = u'301'
RPL_UNAWAY = u'305'
RPL_NOWAWAY = u'306'
RPL_WHOISUSER = u'311'
RPL_WHOISSERVER = u'312'
RPL_WHOISOPERATOR = u'313'
RPL_WHOISIDLE = u'317'
RPL_ENDOFWHOIS = u'318'
RPL_WHOISCHANNELS = u'319'
RPL_WHOWASUSER = u'314'
RPL_ENDOFWHOWAS = u'369'
RPL_LISTSTART = u'321'
RPL_LIST = u'322'
RPL_LISTEND = u'323'
RPL_UNIQOPIS = u'325'
RPL_CHANNELMODEIS = u'324'
RPL_NOTOPIC = u'331'
RPL_TOPIC = u'332'
RPL_INVITING = u'341'
RPL_SUMMONING = u'342'
RPL_INVITELIST = u'346'
RPL_ENDOFINVITELIST = u'347'
RPL_EXCEPTLIST = u'348'
RPL_ENDOFEXCEPTLIST = u'349'
RPL_VERSION = u'351'
RPL_WHOREPLY = u'352'
RPL_ENDOFWHO = u'315'
RPL_NAMREPLY = u'353'
RPL_ENDOFNAMES = u'366'
RPL_LINKS = u'364'
RPL_ENDOFLINKS = u'365'
RPL_BANLIST = u'367'
RPL_ENDOFBANLIST = u'368'
RPL_INFO = u'371'
RPL_ENDOFINFO = u'374'
RPL_MOTDSTART = u'375'
RPL_MOTD = u'372'
RPL_ENDOFMOTD = u'376'
RPL_YOUREOPER = u'381'
RPL_REHASHING = u'382'
RPL_YOURESERVICE = u'383'
RPL_TIME = u'391'
RPL_USERSSTART = u'392'
RPL_USERS = u'393'
RPL_ENDOFUSERS = u'394'
RPL_NOUSERS = u'395'
RPL_TRACELINK = u'200'
RPL_TRACECONNECTING = u'201'
RPL_TRACEHANDSHAKE = u'202'
RPL_TRACEUNKNOWN = u'203'
RPL_TRACEOPERATOR = u'204'
RPL_TRACEUSER = u'205'
RPL_TRACESERVER = u'206'
RPL_TRACESERVICE = u'207'
RPL_TRACENEWTYPE = u'208'
RPL_TRACECLASS = u'209'
RPL_TRACERECONNECT = u'210'
RPL_TRACELOG = u'261'
RPL_TRACEEND = u'262'
RPL_STATSLINKINFO = u'211'
RPL_STATSCOMMANDS = u'212'
RPL_ENDOFSTATS = u'219'
RPL_STATSUPTIME = u'242'
RPL_STATSOLINE = u'243'
RPL_UMODEIS = u'221'
RPL_SERVLIST = u'234'
RPL_SERVLISTEND = u'235'
RPL_LUSERCLIENT = u'251'
RPL_LUSEROP = u'252'
RPL_LUSERUNKNOWN = u'253'
RPL_LUSERCHANNELS = u'254'
RPL_LUSERME = u'255'
RPL_ADMINME = u'256'
RPL_ADMINLOC = u'257'
RPL_ADMINLOC = u'258'
RPL_ADMINEMAIL = u'259'
RPL_TRYAGAIN = u'263'
ERR_NOSUCHNICK = u'401'
ERR_NOSUCHSERVER = u'402'
ERR_NOSUCHCHANNEL = u'403'
ERR_CANNOTSENDTOCHAN = u'404'
ERR_TOOMANYCHANNELS = u'405'
ERR_WASNOSUCHNICK = u'406'
ERR_TOOMANYTARGETS = u'407'
ERR_NOSUCHSERVICE = u'408'
ERR_NOORIGIN = u'409'
ERR_NORECIPIENT = u'411'
ERR_NOTEXTTOSEND = u'412'
ERR_NOTOPLEVEL = u'413'
ERR_WILDTOPLEVEL = u'414'
ERR_BADMASK = u'415'
ERR_UNKNOWNCOMMAND = u'421'
ERR_NOMOTD = u'422'
ERR_NOADMININFO = u'423'
ERR_FILEERROR = u'424'
ERR_NONICKNAMEGIVEN = u'431'
ERR_ERRONEUSNICKNAME = u'432'
ERR_NICKNAMEINUSE = u'433'
ERR_NICKCOLLISION = u'436'
ERR_UNAVAILRESOURCE = u'437'
ERR_USERNOTINCHANNEL = u'441'
ERR_NOTONCHANNEL = u'442'
ERR_USERONCHANNEL = u'443'
ERR_NOLOGIN = u'444'
ERR_SUMMONDISABLED = u'445'
ERR_USERSDISABLED = u'446'
ERR_NOTREGISTERED = u'451'
ERR_NEEDMOREPARAMS = u'461'
ERR_ALREADYREGISTRED = u'462'
ERR_NOPERMFORHOST = u'463'
ERR_PASSWDMISMATCH = u'464'
ERR_YOUREBANNEDCREEP = u'465'
ERR_YOUWILLBEBANNED = u'466'
ERR_KEYSET = u'467'
ERR_CHANNELISFULL = u'471'
ERR_UNKNOWNMODE = u'472'
ERR_INVITEONLYCHAN = u'473'
ERR_BANNEDFROMCHAN = u'474'
ERR_BADCHANNELKEY = u'475'
ERR_BADCHANMASK = u'476'
ERR_NOCHANMODES = u'477'
ERR_BANLISTFULL = u'478'
ERR_NOPRIVILEGES = u'481'
ERR_CHANOPRIVSNEEDED = u'482'
ERR_CANTKILLSERVER = u'483'
ERR_RESTRICTED = u'484'
ERR_UNIQOPPRIVSNEEDED = u'485'
ERR_NOOPERHOST = u'491'
ERR_UMODEUNKNOWNFLAG = u'501'
ERR_USERSDONTMATCH = u'502'
RPL_SERVICEINFO = u'231'
RPL_ENDOFSERVICES = u'232'
RPL_SERVICE = u'233'
RPL_NONE = u'300'
RPL_WHOISCHANOP = u'316'
RPL_KILLDONE = u'361'
RPL_CLOSING = u'362'
RPL_CLOSEEND = u'363'
RPL_INFOSTART = u'373'
RPL_MYPORTIS = u'384'
RPL_STATSCLINE = u'213'
RPL_STATSNLINE = u'214'
RPL_STATSILINE = u'215'
RPL_STATSKLINE = u'216'
RPL_STATSQLINE = u'217'
RPL_STATSYLINE = u'218'
RPL_STATSVLINE = u'240'
RPL_STATSLLINE = u'241'
RPL_STATSHLINE = u'244'
RPL_STATSSLINE = u'244'
RPL_STATSPING = u'246'
RPL_STATSBLINE = u'247'
RPL_STATSDLINE = u'250'
ERR_NOSERVICEHOST = u'492'
